@@ -3,7 +3,7 @@ import logging
 import os
 import shutil
 from pathlib import Path
-from typing import Dict, List, Text, TYPE_CHECKING
+from typing import Dict, Union, List, Text, TYPE_CHECKING
 
 import rasa.shared.core.domain
 from rasa import telemetry
@@ -273,7 +273,11 @@ def validate_stories(args: argparse.Namespace) -> None:
 
 
 def _validate_domain(validator: "Validator") -> bool:
-    return validator.verify_domain_validity()
+    return (
+        validator.verify_domain_validity()
+        and validator.verify_actions_in_stories_rules()
+        and validator.verify_form_slots()
+    )
 
 
 def _validate_nlu(validator: "Validator", args: argparse.Namespace) -> bool:
@@ -376,9 +380,12 @@ def _migrate_responses(args: argparse.Namespace) -> None:
 
 
 async def _convert_to_yaml(
-    out_path: Text, data_path: Text, converter: "TrainingDataConverter"
+    out_path: Text, data_path: Union[list, Text], converter: "TrainingDataConverter"
 ) -> None:
 
+    if isinstance(data_path, list):
+        data_path = data_path[0]
+
     output = Path(out_path)
     if not os.path.exists(output):
         rasa.shared.utils.cli.print_error_and_exit(
