@@ -1,5 +1,4 @@
 import tensorflow as tf
-import tensorflow_addons as tfa
 import numpy as np
 import logging
 import random
@@ -14,40 +13,31 @@ from typing import (
     Any,
 )
 
+from rasa.shared.constants import DIAGNOSTIC_DATA
 from rasa.utils.tensorflow.model_data import RasaModelData, FeatureSignature
 from rasa.utils.tensorflow.constants import (
-    SEQUENCE,
+    LABEL,
     SENTENCE,
+    SEQUENCE_LENGTH,
     RANDOM_SEED,
     EMBEDDING_DIMENSION,
     REGULARIZATION_CONSTANT,
     SIMILARITY_TYPE,
-    WEIGHT_SPARSITY,
-    NUM_TRANSFORMER_LAYERS,
-    TRANSFORMER_SIZE,
-    NUM_HEADS,
-    UNIDIRECTIONAL_ENCODER,
-    KEY_RELATIVE_ATTENTION,
-    VALUE_RELATIVE_ATTENTION,
-    MAX_RELATIVE_POSITION,
+    CONNECTION_DENSITY,
     NUM_NEG,
     LOSS_TYPE,
     MAX_POS_SIM,
     MAX_NEG_SIM,
     USE_MAX_NEG_SIM,
     NEGATIVE_MARGIN_SCALE,
-    HIDDEN_LAYERS_SIZES,
-    DROP_RATE,
-    DENSE_DIMENSION,
-    CONCAT_DIMENSION,
-    DROP_RATE_ATTENTION,
     SCALE_LOSS,
     LEARNING_RATE,
     CONSTRAIN_SIMILARITIES,
     MODEL_CONFIDENCE,
 )
+import rasa.utils.train_utils
 from rasa.utils.tensorflow import layers
-from rasa.utils.tensorflow.transformer import TransformerEncoder
+from rasa.utils.tensorflow import rasa_layers
 from rasa.utils.tensorflow.temp_keras_modules import TmpKerasModel
 from rasa.utils.tensorflow.data_generator import (
     RasaDataGenerator,
@@ -241,14 +231,16 @@ class RasaModel(TmpKerasModel):
         # the list
         return [element_spec]
 
-    def rasa_predict(self, model_data: RasaModelData) -> Dict[Text, tf.Tensor]:
+    def _rasa_predict(
+        self, batch_in: Tuple[np.ndarray]
+    ) -> Dict[Text, Union[np.ndarray, Dict[Text, Any]]]:
         """Custom prediction method that builds tf graph on the first call.
 
         Args:
-            model_data: The model data to use for prediction.
+            batch_in: Prepared batch ready for input to `predict_step` method of model.
 
         Return:
-            Prediction output.
+            Prediction output, including diagnostic data.
         """
         self._training = False
         if not self.prepared_for_prediction:
@@ -257,17 +249,101 @@ class RasaModel(TmpKerasModel):
             self.prepare_for_predict()
             self.prepared_for_prediction = True
 
-        batch_in = RasaBatchDataGenerator.prepare_batch(model_data.data)
-
         if self._run_eagerly:
-            return tf_utils.to_numpy_or_python_type(self.predict_step(batch_in))
+            outputs = tf_utils.to_numpy_or_python_type(self.predict_step(batch_in))
+            if DIAGNOSTIC_DATA in outputs:
+                outputs[DIAGNOSTIC_DATA] = self._empty_lists_to_none_in_dict(
+                    outputs[DIAGNOSTIC_DATA]
+                )
+            return outputs
 
         if self._tf_predict_step is None:
             self._tf_predict_step = tf.function(
                 self.predict_step, input_signature=self._dynamic_signature(batch_in)
             )
 
-        return tf_utils.to_numpy_or_python_type(self._tf_predict_step(batch_in))
+        outputs = tf_utils.to_numpy_or_python_type(self._tf_predict_step(batch_in))
+        if DIAGNOSTIC_DATA in outputs:
+            outputs[DIAGNOSTIC_DATA] = self._empty_lists_to_none_in_dict(
+                outputs[DIAGNOSTIC_DATA]
+            )
+        return outputs
+
+    def run_inference(
+        self, model_data: RasaModelData, batch_size: Union[int, List[int]] = 1
+    ) -> Dict[Text, Union[np.ndarray, Dict[Text, Any]]]:
+        """Implements bulk inferencing through the model.
+
+        Args:
+            model_data: Input data to be fed to the model.
+            batch_size: Size of batches that the generator should create.
+
+        Returns:
+            Model outputs corresponding to the inputs fed.
+        """
+        outputs = {}
+        (data_generator, _,) = rasa.utils.train_utils.create_data_generators(
+            model_data=model_data, batch_sizes=batch_size, epochs=1, shuffle=False,
+        )
+        data_iterator = iter(data_generator)
+        while True:
+            try:
+                # data_generator is a tuple of 2 elements - input and output.
+                # We only need input, since output is always None and not
+                # consumed by our TF graphs.
+                batch_in = next(data_iterator)[0]
+                batch_out = self._rasa_predict(batch_in)
+                outputs = self._merge_batch_outputs(outputs, batch_out)
+            except StopIteration:
+                # Generator ran out of batches, time to finish inferencing
+                break
+        return outputs
+
+    @staticmethod
+    def _merge_batch_outputs(
+        all_outputs: Dict[Text, Union[np.ndarray, Dict[Text, np.ndarray]]],
+        batch_output: Dict[Text, Union[np.ndarray, Dict[Text, np.ndarray]]],
+    ) -> Dict[Text, Union[np.ndarray, Dict[Text, np.ndarray]]]:
+        """Merges a batch's output into the output for all batches.
+
+        Function assumes that the schema of batch output remains the same,
+        i.e. keys and their value types do not change from one batch's
+        output to another.
+
+        Args:
+            all_outputs: Existing output for all previous batches.
+            batch_output: Output for a batch.
+
+        Returns:
+            Merged output with the output for current batch stacked
+            below the output for all previous batches.
+        """
+        if not all_outputs:
+            return batch_output
+        for key, val in batch_output.items():
+            if isinstance(val, np.ndarray):
+                all_outputs[key] = np.concatenate(
+                    [all_outputs[key], batch_output[key]], axis=0
+                )
+
+            elif isinstance(val, dict):
+                # recurse and merge the inner dict first
+                all_outputs[key] = RasaModel._merge_batch_outputs(all_outputs[key], val)
+
+        return all_outputs
+
+    @staticmethod
+    def _empty_lists_to_none_in_dict(input_dict: Dict[Text, Any]) -> Dict[Text, Any]:
+        """Recursively replaces empty list or np array with None in a dictionary."""
+
+        def _recurse(x: Union[Dict[Text, Any], List[Any], np.ndarray]) -> Optional[Any]:
+            if isinstance(x, dict):
+                return {k: _recurse(v) for k, v in x.items()}
+            elif (isinstance(x, list) or isinstance(x, np.ndarray)) and np.size(x) == 0:
+                return None
+            return x
+
+        return _recurse(input_dict)
 
     def _get_metric_results(self, prefix: Optional[Text] = "") -> Dict[Text, float]:
         return {
@@ -327,7 +403,7 @@ class RasaModel(TmpKerasModel):
         # predict on one data example to speed up prediction during inference
         # the first prediction always takes a bit longer to trace tf function
         if not finetune_mode and predict_data_example:
-            model.rasa_predict(predict_data_example)
+            model.run_inference(predict_data_example)
 
         logger.debug("Finished loading the model.")
         return model
@@ -469,6 +545,13 @@ class TransformerRasaModel(RasaModel):
     def _prepare_layers(self) -> None:
         raise NotImplementedError
 
+    def _prepare_label_classification_layers(self, predictor_attribute: Text) -> None:
+        """Prepares layers & loss for the final label prediction step."""
+        self._prepare_embed_layers(predictor_attribute)
+        self._prepare_embed_layers(LABEL)
+
+        self._prepare_dot_product_loss(LABEL, self.config[SCALE_LOSS])
+
     def _prepare_embed_layers(self, name: Text, prefix: Text = "embed") -> None:
         self._tf_layers[f"{prefix}.{name}"] = layers.Embed(
             self.config[EMBEDDING_DIMENSION],
@@ -487,40 +570,10 @@ class TransformerRasaModel(RasaModel):
             layer_sizes,
             drop_rate,
             self.config[REGULARIZATION_CONSTANT],
-            self.config[WEIGHT_SPARSITY],
+            self.config[CONNECTION_DENSITY],
             layer_name_suffix=name,
         )
 
-    def _prepare_transformer_layer(
-        self,
-        name: Text,
-        num_layers: int,
-        units: int,
-        drop_rate: float,
-        drop_rate_attention: float,
-        unidirectional: bool,
-        prefix: Text = "transformer",
-    ) -> None:
-        if num_layers > 0:
-            self._tf_layers[f"{prefix}.{name}"] = TransformerEncoder(
-                num_layers,
-                units,
-                self.config[NUM_HEADS],
-                units * 4,
-                self.config[REGULARIZATION_CONSTANT],
-                dropout_rate=drop_rate,
-                attention_dropout_rate=drop_rate_attention,
-                sparsity=self.config[WEIGHT_SPARSITY],
-                unidirectional=unidirectional,
-                use_key_relative_position=self.config[KEY_RELATIVE_ATTENTION],
-                use_value_relative_position=self.config[VALUE_RELATIVE_ATTENTION],
-                max_relative_position=self.config[MAX_RELATIVE_POSITION],
-                name=f"{name}_encoder",
-            )
-        else:
-            # create lambda so that it can be used later without the check
-            self._tf_layers[f"{prefix}.{name}"] = lambda x, mask, training: (x, None)
-
     def _prepare_dot_product_loss(
         self, name: Text, scale_loss: bool, prefix: Text = "loss"
     ) -> None:
@@ -537,89 +590,6 @@ class TransformerRasaModel(RasaModel):
             model_confidence=self.config[MODEL_CONFIDENCE],
         )
 
-    def _prepare_sparse_dense_dropout_layers(
-        self, name: Text, drop_rate: float
-    ) -> None:
-        self._tf_layers[f"sparse_input_dropout.{name}"] = layers.SparseDropout(
-            rate=drop_rate
-        )
-        self._tf_layers[f"dense_input_dropout.{name}"] = tf.keras.layers.Dropout(
-            rate=drop_rate
-        )
-
-    def _prepare_sparse_dense_layers(
-        self, data_signature: List[FeatureSignature], name: Text, dense_dim: int
-    ) -> None:
-        sparse = False
-        dense = False
-        for is_sparse, _, _ in data_signature:
-            if is_sparse:
-                sparse = True
-            else:
-                dense = True
-
-        if sparse:
-            self._tf_layers[f"sparse_to_dense.{name}"] = layers.DenseForSparse(
-                units=dense_dim,
-                reg_lambda=self.config[REGULARIZATION_CONSTANT],
-                name=name,
-            )
-            if not dense:
-                # create dense labels for the input to use in negative sampling
-                self._tf_layers[f"sparse_to_dense_ids.{name}"] = layers.DenseForSparse(
-                    units=2,
-                    use_bias=False,
-                    trainable=False,
-                    name=f"sparse_to_dense_ids.{name}",
-                )
-
-    def _prepare_input_layers(self, name: Text) -> None:
-        self._prepare_ffnn_layer(
-            name, self.config[HIDDEN_LAYERS_SIZES][name], self.config[DROP_RATE]
-        )
-
-        for feature_type in [SENTENCE, SEQUENCE]:
-            if (
-                name not in self.data_signature
-                or feature_type not in self.data_signature[name]
-            ):
-                continue
-
-            self._prepare_sparse_dense_dropout_layers(
-                f"{name}_{feature_type}", self.config[DROP_RATE]
-            )
-            self._prepare_sparse_dense_layers(
-                self.data_signature[name][feature_type],
-                f"{name}_{feature_type}",
-                self.config[DENSE_DIMENSION][name],
-            )
-            self._prepare_ffnn_layer(
-                f"{name}_{feature_type}",
-                [self.config[CONCAT_DIMENSION][name]],
-                self.config[DROP_RATE],
-                prefix="concat_layer",
-            )
-
-    def _prepare_sequence_layers(self, name: Text) -> None:
-        self._prepare_input_layers(name)
-
-        size = self.config[TRANSFORMER_SIZE]
-        if isinstance(size, dict):
-            size = size[name]
-
-        num_layers = self.config[NUM_TRANSFORMER_LAYERS]
-        if isinstance(num_layers, dict):
-            num_layers = num_layers[name]
-
-        self._prepare_transformer_layer(
-            name,
-            num_layers,
-            size,
-            self.config[DROP_RATE],
-            self.config[DROP_RATE_ATTENTION],
-            self.config[UNIDIRECTIONAL_ENCODER],
-        )
-
     def _prepare_entity_recognition_layers(self) -> None:
         for tag_spec in self._entity_tag_specs:
             name = tag_spec.tag_name
@@ -636,201 +606,6 @@ class TransformerRasaModel(RasaModel):
                 f"tags.{name}",
             )
 
-    def _combine_sparse_dense_features(
-        self,
-        features: List[Union[np.ndarray, tf.Tensor, tf.SparseTensor]],
-        name: Text,
-        mask: Optional[tf.Tensor] = None,
-        sparse_dropout: bool = False,
-        dense_dropout: bool = False,
-    ) -> Optional[tf.Tensor]:
-        if not features:
-            return None
-
-        dense_features = []
-
-        for f in features:
-            if isinstance(f, tf.SparseTensor):
-                if sparse_dropout:
-                    _f = self._tf_layers[f"sparse_input_dropout.{name}"](
-                        f, self._training
-                    )
-                else:
-                    _f = f
-
-                dense_f = self._tf_layers[f"sparse_to_dense.{name}"](_f)
-
-                if dense_dropout:
-                    dense_f = self._tf_layers[f"dense_input_dropout.{name}"](
-                        dense_f, self._training
-                    )
-
-                dense_features.append(dense_f)
-            else:
-                dense_features.append(f)
-
-        if mask is None:
-            return tf.concat(dense_features, axis=-1)
-
-        return tf.concat(dense_features, axis=-1) * mask
-
-    def _combine_sequence_sentence_features(
-        self,
-        sequence_features: List[Union[tf.Tensor, tf.SparseTensor]],
-        sentence_features: List[Union[tf.Tensor, tf.SparseTensor]],
-        mask_sequence: tf.Tensor,
-        mask_text: tf.Tensor,
-        name: Text,
-        sparse_dropout: bool = False,
-        dense_dropout: bool = False,
-    ) -> tf.Tensor:
-        sequence_x = self._combine_sparse_dense_features(
-            sequence_features,
-            f"{name}_{SEQUENCE}",
-            mask_sequence,
-            sparse_dropout,
-            dense_dropout,
-        )
-        sentence_x = self._combine_sparse_dense_features(
-            sentence_features, f"{name}_{SENTENCE}", None, sparse_dropout, dense_dropout
-        )
-
-        if sequence_x is not None and sentence_x is None:
-            return sequence_x
-
-        if sequence_x is None and sentence_x is not None:
-            return sentence_x
-
-        if sequence_x is not None and sentence_x is not None:
-            return self._concat_sequence_sentence_features(
-                sequence_x, sentence_x, name, mask_text
-            )
-
-        raise ValueError(
-            "No features are present. Please check your configuration file."
-        )
-
-    def _concat_sequence_sentence_features(
-        self,
-        sequence_x: tf.Tensor,
-        sentence_x: tf.Tensor,
-        name: Text,
-        mask_text: tf.Tensor,
-    ) -> tf.Tensor:
-        if sequence_x.shape[-1] != sentence_x.shape[-1]:
-            sequence_x = self._tf_layers[f"concat_layer.{name}_{SEQUENCE}"](
-                sequence_x, self._training
-            )
-            sentence_x = self._tf_layers[f"concat_layer.{name}_{SENTENCE}"](
-                sentence_x, self._training
-            )
-
-        # we need to concatenate the sequence features with the sentence features
-        # we cannot use tf.concat as the sequence features are padded
-
-        # (1) get position of sentence features in mask
-        last = mask_text * tf.math.cumprod(
-            1 - mask_text, axis=1, exclusive=True, reverse=True
-        )
-        # (2) multiply by sentence features so that we get a matrix of
-        #     batch-dim x seq-dim x feature-dim with zeros everywhere except for
-        #     for the sentence features
-        sentence_x = last * sentence_x
-
-        # (3) add a zero to the end of sequence matrix to match the final shape
-        sequence_x = tf.pad(sequence_x, [[0, 0], [0, 1], [0, 0]])
-
-        # (4) sum up sequence features and sentence features
-        return sequence_x + sentence_x
-
-    def _features_as_seq_ids(
-        self, features: List[Union[np.ndarray, tf.Tensor, tf.SparseTensor]], name: Text
-    ) -> Optional[tf.Tensor]:
-        """Creates dense labels for negative sampling."""
-        # if there are dense features - we can use them
-        for f in features:
-            if not isinstance(f, tf.SparseTensor):
-                seq_ids = tf.stop_gradient(f)
-                # add a zero to the seq dimension for the sentence features
-                seq_ids = tf.pad(seq_ids, [[0, 0], [0, 1], [0, 0]])
-                return seq_ids
-
-        # use additional sparse to dense layer
-        for f in features:
-            if isinstance(f, tf.SparseTensor):
-                seq_ids = tf.stop_gradient(
-                    self._tf_layers[f"sparse_to_dense_ids.{name}"](f)
-                )
-                # add a zero to the seq dimension for the sentence features
-                seq_ids = tf.pad(seq_ids, [[0, 0], [0, 1], [0, 0]])
-                return seq_ids
-
-        return None
-
-    def _create_sequence(
-        self,
-        sequence_features: List[Union[tf.Tensor, tf.SparseTensor]],
-        sentence_features: List[Union[tf.Tensor, tf.SparseTensor]],
-        mask_sequence: tf.Tensor,
-        mask: tf.Tensor,
-        name: Text,
-        sparse_dropout: bool = False,
-        dense_dropout: bool = False,
-        masked_lm_loss: bool = False,
-        sequence_ids: bool = False,
-    ) -> Tuple[
-        tf.Tensor,
-        tf.Tensor,
-        Optional[tf.Tensor],
-        Optional[tf.Tensor],
-        Optional[tf.Tensor],
-    ]:
-        if sequence_ids:
-            seq_ids = self._features_as_seq_ids(sequence_features, f"{name}_{SEQUENCE}")
-        else:
-            seq_ids = None
-
-        inputs = self._combine_sequence_sentence_features(
-            sequence_features,
-            sentence_features,
-            mask_sequence,
-            mask,
-            name,
-            sparse_dropout,
-            dense_dropout,
-        )
-        inputs = self._tf_layers[f"ffnn.{name}"](inputs, self._training)
-
-        if masked_lm_loss:
-            transformer_inputs, lm_mask_bool = self._tf_layers[f"{name}_input_mask"](
-                inputs, mask, self._training
-            )
-        else:
-            transformer_inputs = inputs
-            lm_mask_bool = None
-
-        outputs, attention_weights = self._tf_layers[f"transformer.{name}"](
-            transformer_inputs, 1 - mask, self._training
-        )
-
-        if isinstance(self.config[NUM_TRANSFORMER_LAYERS], int):
-            num_layers = self.config[NUM_TRANSFORMER_LAYERS]
-        else:
-            num_layers = self.config[NUM_TRANSFORMER_LAYERS][name]
-
-        if num_layers > 0:
-            # apply activation
-            outputs = tfa.activations.gelu(outputs)
-
-        return outputs, inputs, seq_ids, lm_mask_bool, attention_weights
-
-    @staticmethod
-    def _compute_mask(sequence_lengths: tf.Tensor) -> tf.Tensor:
-        mask = tf.sequence_mask(sequence_lengths, dtype=tf.float32)
-        # explicitly add last dimension to mask
-        # to track correctly dynamic sequences
-        return tf.expand_dims(mask, -1)
-
     @staticmethod
     def _last_token(x: tf.Tensor, sequence_lengths: tf.Tensor) -> tf.Tensor:
         last_sequence_index = tf.maximum(0, sequence_lengths - 1)
@@ -849,30 +624,47 @@ class TransformerRasaModel(RasaModel):
             return None
 
         sequence_lengths = tf.cast(tf_batch_data[key][sub_key][0], dtype=tf.int32)
-        return self._compute_mask(sequence_lengths)
+        return rasa_layers.compute_mask(sequence_lengths)
 
-    @staticmethod
-    def _get_sequence_lengths(
-        tf_batch_data: Dict[Text, Dict[Text, List[tf.Tensor]]],
-        key: Text,
-        sub_key: Text,
-        batch_dim: int = 1,
+    def _get_sequence_feature_lengths(
+        self, tf_batch_data: Dict[Text, Dict[Text, List[tf.Tensor]]], key: Text
     ) -> tf.Tensor:
-        # sentence features have a sequence lengths of 1
-        # if sequence features are present we add the sequence lengths of those
+        """Fetches the sequence lengths of real tokens per input example.
 
-        sequence_lengths = tf.ones([batch_dim], dtype=tf.int32)
-        if key in tf_batch_data and sub_key in tf_batch_data[key]:
-            sequence_lengths += tf.cast(tf_batch_data[key][sub_key][0], dtype=tf.int32)
+        The number of real tokens for an example is the same as the length of the
+        sequence of the sequence-level (token-level) features for that input example.
+        """
+        if key in tf_batch_data and SEQUENCE_LENGTH in tf_batch_data[key]:
+            return tf.cast(tf_batch_data[key][SEQUENCE_LENGTH][0], dtype=tf.int32)
 
-        return tf.cast(tf_batch_data[key][sub_key][0], dtype=tf.int32) + 1
+        batch_dim = self._get_batch_dim(tf_batch_data[key])
+        return tf.zeros([batch_dim], dtype=tf.int32)
+
+    def _get_sentence_feature_lengths(
+        self, tf_batch_data: Dict[Text, Dict[Text, List[tf.Tensor]]], key: Text,
+    ) -> tf.Tensor:
+        """Fetches the sequence lengths of sentence-level features per input example.
+
+        This is needed because we treat sentence-level features as token-level features
+        with 1 token per input example. Hence, the sequence lengths returned by this
+        function are all 1s if sentence-level features are present, and 0s otherwise.
+        """
+        batch_dim = self._get_batch_dim(tf_batch_data[key])
+
+        if key in tf_batch_data and SENTENCE in tf_batch_data[key]:
+            return tf.ones([batch_dim], dtype=tf.int32)
+
+        return tf.zeros([batch_dim], dtype=tf.int32)
 
     @staticmethod
     def _get_batch_dim(attribute_data: Dict[Text, List[tf.Tensor]]) -> int:
-        if SEQUENCE in attribute_data:
-            return tf.shape(attribute_data[SEQUENCE][0])[0]
-
-        return tf.shape(attribute_data[SENTENCE][0])[0]
+        # All the values in the attribute_data dict should be lists of tensors, each
+        # tensor of the shape (batch_dim, ...). So we take the first non-empty list we
+        # encounter and infer the batch size from its first tensor.
+        for key, data in attribute_data.items():
+            if data:
+                return tf.shape(data[0])[0]
+        return None
 
     def _calculate_entity_loss(
         self,
